<?php
/*
Plugin Name: Easy Order Print
Plugin URI: https://cs-cart-soft.eu
Description: Easy Order Print - Print your WooCommerce orders automatically to any Thermal printer or to any printer. You need to Download Windows Application. 
Author: devteamsipware
Version: 0.2
Author URI: 
*/

// add custom REST API GET endpoint /easyprint/order/<id>
add_action('rest_api_init', function () {
    register_rest_route('easyprint', '/orders/(?P<id>\d+)', array(
        'methods' => 'GET',
        'callback' => 'update_order',
        'permission_callback' => 'authentication_callback',
    ));
});

// update WooCommerce order
function update_order($request) {
    $order_id = $request['id'];
    $order = wc_get_order($order_id);

    if (!$order) {
        echo "NO_ORDER";
        die; 
    }

    // Attempt to update the order status
    try {
        $order->update_status('completed');
        // $success_response = array(
        //     'message' => 'Order updated successfully.',
        //     'data' => array(
        //         'order_id' => $order_id
        //     )
        // );
        // return new WP_REST_Response($success_response, 200); // Return success with 200 OK status code
        echo "Order_status_has_been_changed";		   
        die();
    } catch (Exception $e) {
        // $error_response = array(
        //     'message' => 'Failed to update order.',
        //     'data' => array(
        //         'order_id' => $order_id,
        //         'error' => $e->getMessage()
        //     )
        // );
        // return new WP_REST_Response($error_response, 500); // Return error with 500 Internal Server Error status code
        echo "Failed_to_update_order";
    }
}


// add custom REST API GET endpoint /easyprint/orders
add_action('rest_api_init', function () {
    register_rest_route('easyprint', '/orders', array(
        'methods' => 'GET',
        'callback' => 'get_new_orders',
        'permission_callback' => 'authentication_callback',
    ));
});


function remove_not_need_symbols($orderdata){
	$newdata = "";
	$newdata = str_replace("*","", $orderdata);
	$newdata = str_replace("~","", $newdata);
	$newdata = str_replace("%","", $newdata);
	$newdata = str_replace("#","", $newdata);
	$newdata = str_replace("&","", $newdata);
	$newdata = str_replace("$","", $newdata);
	$newdata = str_replace("^","", $newdata);
	$newdata = str_replace("{","", $newdata);
	$newdata = str_replace("}","", $newdata);
   return $newdata; 
}


function get_new_orders() {
	$string_to_send = "";
   	//Request orders
  	//Order status: pending, processing,on-hold,completed,cancelled,refunded,failed,trash
    
    // Get pending orders from woocommerce
    $orders = wc_get_orders(array(
        'status' => 'processing',
    ));
    
    if (empty($orders)) {
        echo "NO_ORDER";
        die;
    }
    
    
    $string_to_send = '';
    //foreach ($orders as $order){
    $order_arr = $orders[0]->get_data();

    /* 
    //---Debug----
    echo "<pre>";
    print_r($order_arr);
    echo "</pre>"; 
    */
    
    
    $order_id = trim($order_arr['id']);
    $string_to_send .= "#" .$order_id;
            
  
        
    foreach($order_arr['line_items'] as $prod){
    	$name = $prod['name'];
        $name = remove_not_need_symbols($name);
        $quantity = $prod['quantity'];
        $product_cost = $prod['total'];
        $string_to_send .= "*" . $name . "~" . $quantity . "~";
        $metadata = $prod->get_meta_data(); 
        $number_of_product_options = sizeof($metadata);
        $string_to_send .= $number_of_product_options."~".$product_cost."~";
         
         /*
         echo '-----Debug-----';
         echo "<pre>";
         print_r($metadata);
         echo "</pre>";
         echo "----------------------------<br>";
         echo $number_of_product_options;
         echo '-----Debug End-----';           
         */
         foreach ($metadata as $row){
         	 $option_arr = $row->get_data();
             //echo "<br>".$option_arr['key']. "==>>".$option_arr['value'];   
             $option_name = $option_arr['key'];
             $option_name = remove_not_need_symbols($option_name);
             $option_value = $option_arr['value'];
             $option_value = remove_not_need_symbols($option_value);
             $string_to_send .= $option_name . "~" . $option_value . "~";
          }       
       }
               
       // Get Shipping Address
       $string_to_send .= "%";
       $shipping_address = $order_arr['shipping'];
        
       foreach ($shipping_address as $key => $value) {
            $string_to_send .= remove_not_need_symbols($value) . "&";
       }

       $string_to_send = rtrim($string_to_send, "&");
       $string_to_send .= "%";
             
       $string_to_send .= "$";
              
       // Get Payment method
       $payment_method = $order_arr['payment_method'];
       $payment_method = remove_not_need_symbols($payment_method);
       $payment_method_title = $order_arr['payment_method_title'];
       $payment_method_title = remove_not_need_symbols($payment_method_title);
        
       if (empty($payment_method)) {
            $payment_method = "No value";
       }
       if (empty($payment_method_title)) {
           $payment_method_title = "No Value";
       }
       
       $string_to_send .= $payment_method . "~" . $payment_method_title . "$";
        
      
             
       // Get Customer note
       $customer_note = $order_arr['customer_note'];
       $customer_note = remove_not_need_symbols($customer_note);
        
       if (empty($customer_note)) {
            $customer_note = "No value";
       }
  
       $string_to_send = count($order_arr['line_items']) . $string_to_send . "~" . $customer_note . "~";
    
       //echo $string_to_send;
       echo esc_html($string_to_send);
              

}


// callback to ensure the caller has a valid Bearer Token
function authentication_callback($request) {
  
	
	// Check if the current user has a valid bearer token
    $token = $request->get_header('Authorization');
    // echo "Received Token ==>".$token."<br>";
   
    if (!$token || !preg_match('/Bearer\s(\S+)/', $token, $matches)) {
        return new WP_Error('rest_forbidden', esc_html__('Unauthorized Access Code:01'), array('status' => 401));
    }

     
    // Here you validate the token against your authentication mechanism
    // retrieve value from options
    $easyprint_options = get_option( 'easyprint_option_name' ); // Array of All Options
	$api_token_0 = $easyprint_options['api_token_0']; // API Token
	
	$token = str_replace("Bearer","",$token);
	$token = trim($token);
	
	
	if ($token == $api_token_0) {
	    $valid_token = true;
    }

    
    
    if (!$valid_token) {
        return new WP_Error('rest_forbidden', esc_html__('Unauthorized Access Code:02'), array('status' => 401));
    }

    // Token is valid, return true to allow access to the endpoint
    return true;
}


/**
 * Plugin options page (found under Settings)
 * Generated by the WordPress Option Page generator
 * at http://jeremyhixon.com/wp-tools/option-page/
 */
class EasyPrint {
	private $easyprint_options;

	public function __construct() {
		add_action( 'admin_menu', array( $this, 'easyprint_add_plugin_page' ) );
		add_action( 'admin_init', array( $this, 'easyprint_page_init' ) );
		
		// Call activation hook to initialize options upon plugin activation
        register_activation_hook( __FILE__, array( $this, 'easyprint_plugin_activation' ) );
    }
    
    //function to run upon plugin activation
    public function easyprint_plugin_activation() {
        // Generate a random string for api_token_0
        $api_token = wp_generate_password( 32, false );

        // Initialize options array
        $easyprint_options = array(
            'api_token_0' => $api_token
            // Add more options here if needed
        );

        // Add the options to the database
        add_option( 'easyprint_option_name', $easyprint_options );
    }
	

	public function easyprint_add_plugin_page() {
		add_options_page(
			'Easy Order Print', // page_title
			'EasyPrint', // menu_title
			'manage_options', // capability
			'easyprint', // menu_slug
			array( $this, 'easyprint_create_admin_page' ) // function
		);
	}

	public function easyprint_create_admin_page() {
		$this->easyprint_options = get_option( 'easyprint_option_name' ); ?>

		<div class="wrap">
			<h2>Easy Order Print</h2>
			<p></p>
			<?php settings_errors(); ?>

			<form method="post" action="options.php">
				<?php
					settings_fields( 'easyprint_option_group' );
					do_settings_sections( 'easyprint-admin' );
					submit_button();
				?>
			</form>
		</div>
	<?php }

	public function easyprint_page_init() {
		register_setting(
			'easyprint_option_group', // option_group
			'easyprint_option_name', // option_name
			array( $this, 'easyprint_sanitize' ) // sanitize_callback
		);

		add_settings_section(
			'easyprint_setting_section', // id
			'Settings', // title
			array( $this, 'easyprint_section_info' ), // callback
			'easyprint-admin' // page
		);

		add_settings_field(
			'api_token_0', // id
			'API Token', // title
			array( $this, 'api_token_0_callback' ), // callback
			'easyprint-admin', // page
			'easyprint_setting_section' // section
		);
	}

	public function easyprint_sanitize($input) {
		$sanitary_values = array();
		if ( isset( $input['api_token_0'] ) ) {
			$sanitary_values['api_token_0'] = sanitize_text_field( $input['api_token_0'] );
		}

		return $sanitary_values;
	}

	public function easyprint_section_info() {
		
	}

	public function api_token_0_callback() {
    	// Get the saved API token value from options, or generate a random string as default
    	$api_token = isset($this->easyprint_options['api_token_0']) ? esc_attr($this->easyprint_options['api_token_0']) : wp_generate_password(32, false);

	    printf(
    	    '<input class="regular-text" type="text" name="easyprint_option_name[api_token_0]" id="api_token_0" value="%s">',
        	esc_html($api_token)
    	);
	}

} // of class EasyPrint

if ( is_admin() )
	$easyprint = new EasyPrint();

?>